/**
 * WS-Attacker - A Modular Web Services Penetration Testing Framework Copyright
 * (C) 2013 Christian Altmeier
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */
package wsattacker.plugin.intelligentdos.ui.dialog;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import org.apache.log4j.Logger;
import wsattacker.library.intelligentdos.dos.CoerciveParsing;
import wsattacker.library.intelligentdos.dos.DoSAttack;
import wsattacker.library.intelligentdos.dos.HashCollision;
import wsattacker.library.intelligentdos.dos.XmlAttributeCount;
import wsattacker.library.intelligentdos.dos.XmlElementCount;
import wsattacker.library.intelligentdos.dos.XmlEntityExpansion;
import wsattacker.library.intelligentdos.dos.XmlExternalEntity;
import wsattacker.library.intelligentdos.dos.XmlOverlongNames;
import wsattacker.plugin.intelligentdos.ui.dos.ConfigureCoerceiveParsingDialog_NB;
import wsattacker.plugin.intelligentdos.ui.dos.ConfigureHashCollisionDialog_NB;
import wsattacker.plugin.intelligentdos.ui.dos.ConfigureXmlAttributeCountDialog_NB;
import wsattacker.plugin.intelligentdos.ui.dos.ConfigureXmlElementCountDialog_NB;
import wsattacker.plugin.intelligentdos.ui.dos.ConfigureXmlEntityExpansionDialog_NB;
import wsattacker.plugin.intelligentdos.ui.dos.ConfigureXmlExternalEntityDialog_NB;
import wsattacker.plugin.intelligentdos.ui.dos.ConfigureXmlOverlongNamesDialog_NB;

/**
 * @author Christian Altmeier
 */
public class ConfigureAttacksDialog_NB
    extends javax.swing.JDialog
{

    private final DoSAttack[] defaultAttacks = { new CoerciveParsing(), new XmlElementCount(), new XmlAttributeCount(),
        new XmlEntityExpansion(), new XmlExternalEntity(), new HashCollision(), new XmlOverlongNames() };

    private final Map<JCheckBox, DoSAttack> map = new HashMap<JCheckBox, DoSAttack>();

    private final Map<JButton, DoSAttack> mapBtn = new HashMap<JButton, DoSAttack>();

    private static final Logger logger = Logger.getLogger( ConfigureAttacksDialog_NB.class );

    private boolean ok;

    /**
     * Creates new form ConfigureAttacksDialog_NB
     * 
     * @param attacks
     */
    public ConfigureAttacksDialog_NB( DoSAttack[] attacks )
    {
        // set the title and center the dialog
        setTitle( "Configure DoS Attacks" );

        initComponents();

        JCheckBox[] checkboxes = { cbCoPa, cbElCo, cbAtCo, cbEnEx, cbExEn, cbHaCo, cbOvNa };
        JButton[] buttons = { btnCoPa, btnElCo, btnAtCo, btnEnEx, btnExEn, btnHaCo, btnOvNa };

        for ( int i = 0; i < defaultAttacks.length; i++ )
        {
            boolean found = false;
            for ( DoSAttack current : attacks )
            {
                if ( defaultAttacks[i].getClass().equals( current.getClass() ) )
                {
                    checkboxes[i].setSelected( true );
                    map.put( checkboxes[i], current );
                    mapBtn.put( buttons[i], current );
                    found = true;
                }
            }

            if ( !found )
            {
                map.put( checkboxes[i], defaultAttacks[i] );
                mapBtn.put( buttons[i], defaultAttacks[i] );
                buttons[i].setEnabled( false );
            }
        }

    }

    /**
     * This method is called from within the constructor to initialize the form. WARNING: Do NOT modify this code. The
     * content of this method is always regenerated by the Form Editor.
     */
    @SuppressWarnings( "unchecked" )
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents()
    {

        cbCoPa = new javax.swing.JCheckBox();
        cbElCo = new javax.swing.JCheckBox();
        cbAtCo = new javax.swing.JCheckBox();
        cbEnEx = new javax.swing.JCheckBox();
        cbExEn = new javax.swing.JCheckBox();
        cbHaCo = new javax.swing.JCheckBox();
        cbOvNa = new javax.swing.JCheckBox();
        btnCoPa = new javax.swing.JButton();
        btnElCo = new javax.swing.JButton();
        btnAtCo = new javax.swing.JButton();
        btnEnEx = new javax.swing.JButton();
        btnExEn = new javax.swing.JButton();
        btnHaCo = new javax.swing.JButton();
        btnOvNa = new javax.swing.JButton();
        btnOk = new javax.swing.JButton();

        setDefaultCloseOperation( javax.swing.WindowConstants.DISPOSE_ON_CLOSE );

        cbCoPa.setText( "CoerciveParsing" );
        cbCoPa.addActionListener( new java.awt.event.ActionListener()
        {
            @Override
            public void actionPerformed( java.awt.event.ActionEvent evt )
            {
                cbCoPaActionPerformed( evt );
            }
        } );

        cbElCo.setText( "XMLElementCount" );
        cbElCo.addActionListener( new java.awt.event.ActionListener()
        {
            @Override
            public void actionPerformed( java.awt.event.ActionEvent evt )
            {
                cbElCoActionPerformed( evt );
            }
        } );

        cbAtCo.setText( "XMLAttributeCount" );
        cbAtCo.addActionListener( new java.awt.event.ActionListener()
        {
            @Override
            public void actionPerformed( java.awt.event.ActionEvent evt )
            {
                cbAtCoActionPerformed( evt );
            }
        } );

        cbEnEx.setText( "XMLEntityExpansion" );
        cbEnEx.addActionListener( new java.awt.event.ActionListener()
        {
            @Override
            public void actionPerformed( java.awt.event.ActionEvent evt )
            {
                cbEnExActionPerformed( evt );
            }
        } );

        cbExEn.setText( "XMLExternalEntity" );
        cbExEn.addActionListener( new java.awt.event.ActionListener()
        {
            @Override
            public void actionPerformed( java.awt.event.ActionEvent evt )
            {
                cbExEnActionPerformed( evt );
            }
        } );

        cbHaCo.setText( "HashCollision" );
        cbHaCo.addActionListener( new java.awt.event.ActionListener()
        {
            @Override
            public void actionPerformed( java.awt.event.ActionEvent evt )
            {
                cbHaCoActionPerformed( evt );
            }
        } );

        cbOvNa.setText( "XMLOverlog" );
        cbOvNa.addActionListener( new java.awt.event.ActionListener()
        {
            @Override
            public void actionPerformed( java.awt.event.ActionEvent evt )
            {
                cbOvNaActionPerformed( evt );
            }
        } );

        btnCoPa.setText( "configure" );
        btnCoPa.addActionListener( new java.awt.event.ActionListener()
        {
            @Override
            public void actionPerformed( java.awt.event.ActionEvent evt )
            {
                btnCoPaActionPerformed( evt );
            }
        } );

        btnElCo.setText( "configure" );
        btnElCo.addActionListener( new java.awt.event.ActionListener()
        {
            @Override
            public void actionPerformed( java.awt.event.ActionEvent evt )
            {
                btnElCoActionPerformed( evt );
            }
        } );

        btnAtCo.setText( "configure" );
        btnAtCo.addActionListener( new java.awt.event.ActionListener()
        {
            @Override
            public void actionPerformed( java.awt.event.ActionEvent evt )
            {
                btnAtCoActionPerformed( evt );
            }
        } );

        btnEnEx.setText( "configure" );
        btnEnEx.addActionListener( new java.awt.event.ActionListener()
        {
            @Override
            public void actionPerformed( java.awt.event.ActionEvent evt )
            {
                btnEnExActionPerformed( evt );
            }
        } );

        btnExEn.setText( "configure" );
        btnExEn.addActionListener( new java.awt.event.ActionListener()
        {
            @Override
            public void actionPerformed( java.awt.event.ActionEvent evt )
            {
                btnExEnActionPerformed( evt );
            }
        } );

        btnHaCo.setText( "configure" );
        btnHaCo.addActionListener( new java.awt.event.ActionListener()
        {
            @Override
            public void actionPerformed( java.awt.event.ActionEvent evt )
            {
                btnHaCoActionPerformed( evt );
            }
        } );

        btnOvNa.setText( "configure" );
        btnOvNa.addActionListener( new java.awt.event.ActionListener()
        {
            @Override
            public void actionPerformed( java.awt.event.ActionEvent evt )
            {
                btnOvNaActionPerformed( evt );
            }
        } );

        btnOk.setText( "OK" );
        btnOk.addActionListener( new java.awt.event.ActionListener()
        {
            @Override
            public void actionPerformed( java.awt.event.ActionEvent evt )
            {
                btnOkActionPerformed( evt );
            }
        } );

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout( getContentPane() );
        getContentPane().setLayout( layout );
        layout.setHorizontalGroup( layout.createParallelGroup( javax.swing.GroupLayout.Alignment.LEADING ).addGroup( layout.createSequentialGroup().addContainerGap().addGroup( layout.createParallelGroup( javax.swing.GroupLayout.Alignment.LEADING ).addGroup( layout.createParallelGroup( javax.swing.GroupLayout.Alignment.TRAILING,
                                                                                                                                                                                                                                                                                              false ).addGroup( javax.swing.GroupLayout.Alignment.LEADING,
                                                                                                                                                                                                                                                                                                                layout.createSequentialGroup().addComponent( cbOvNa ).addPreferredGap( javax.swing.LayoutStyle.ComponentPlacement.RELATED,
                                                                                                                                                                                                                                                                                                                                                                                       javax.swing.GroupLayout.DEFAULT_SIZE,
                                                                                                                                                                                                                                                                                                                                                                                       Short.MAX_VALUE ).addComponent( btnOvNa ) ).addGroup( layout.createSequentialGroup().addComponent( cbHaCo ).addPreferredGap( javax.swing.LayoutStyle.ComponentPlacement.RELATED,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    javax.swing.GroupLayout.DEFAULT_SIZE,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    Short.MAX_VALUE ).addComponent( btnHaCo ) ).addGroup( javax.swing.GroupLayout.Alignment.LEADING,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          layout.createSequentialGroup().addComponent( cbExEn ).addPreferredGap( javax.swing.LayoutStyle.ComponentPlacement.RELATED,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                 javax.swing.GroupLayout.DEFAULT_SIZE,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                 Short.MAX_VALUE ).addComponent( btnExEn ) ).addGroup( layout.createSequentialGroup().addComponent( cbCoPa ).addPreferredGap( javax.swing.LayoutStyle.ComponentPlacement.RELATED,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                              javax.swing.GroupLayout.DEFAULT_SIZE,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                              Short.MAX_VALUE ).addComponent( btnCoPa ) ).addGroup( javax.swing.GroupLayout.Alignment.LEADING,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    layout.createSequentialGroup().addComponent( cbElCo ).addPreferredGap( javax.swing.LayoutStyle.ComponentPlacement.RELATED,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                           javax.swing.GroupLayout.DEFAULT_SIZE,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                           Short.MAX_VALUE ).addComponent( btnElCo ) ).addGroup( javax.swing.GroupLayout.Alignment.LEADING,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                 layout.createSequentialGroup().addComponent( cbAtCo ).addPreferredGap( javax.swing.LayoutStyle.ComponentPlacement.RELATED,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        javax.swing.GroupLayout.DEFAULT_SIZE,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        Short.MAX_VALUE ).addComponent( btnAtCo ) ).addGroup( javax.swing.GroupLayout.Alignment.LEADING,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                              layout.createSequentialGroup().addComponent( cbEnEx ).addGap( 18,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                            18,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                            18 ).addComponent( btnEnEx ) ) ).addComponent( btnOk ) ).addContainerGap( javax.swing.GroupLayout.DEFAULT_SIZE,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      Short.MAX_VALUE ) ) );
        layout.setVerticalGroup( layout.createParallelGroup( javax.swing.GroupLayout.Alignment.LEADING ).addGroup( layout.createSequentialGroup().addContainerGap().addGroup( layout.createParallelGroup( javax.swing.GroupLayout.Alignment.BASELINE ).addComponent( btnCoPa ).addComponent( cbCoPa ) ).addPreferredGap( javax.swing.LayoutStyle.ComponentPlacement.RELATED ).addGroup( layout.createParallelGroup( javax.swing.GroupLayout.Alignment.BASELINE ).addComponent( cbElCo ).addComponent( btnElCo ) ).addGap( 4,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          4,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          4 ).addGroup( layout.createParallelGroup( javax.swing.GroupLayout.Alignment.BASELINE ).addComponent( cbAtCo ).addComponent( btnAtCo ) ).addPreferredGap( javax.swing.LayoutStyle.ComponentPlacement.RELATED ).addGroup( layout.createParallelGroup( javax.swing.GroupLayout.Alignment.BASELINE ).addComponent( btnEnEx ).addComponent( cbEnEx ) ).addPreferredGap( javax.swing.LayoutStyle.ComponentPlacement.RELATED ).addGroup( layout.createParallelGroup( javax.swing.GroupLayout.Alignment.BASELINE ).addComponent( btnExEn ).addComponent( cbExEn ) ).addGap( 3,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                              3,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                              3 ).addGroup( layout.createParallelGroup( javax.swing.GroupLayout.Alignment.BASELINE ).addComponent( cbHaCo ).addComponent( btnHaCo ) ).addPreferredGap( javax.swing.LayoutStyle.ComponentPlacement.RELATED ).addGroup( layout.createParallelGroup( javax.swing.GroupLayout.Alignment.BASELINE ).addComponent( cbOvNa ).addComponent( btnOvNa ) ).addPreferredGap( javax.swing.LayoutStyle.ComponentPlacement.UNRELATED ).addComponent( btnOk ).addContainerGap( javax.swing.GroupLayout.DEFAULT_SIZE,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                               Short.MAX_VALUE ) ) );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void btnOvNaActionPerformed( java.awt.event.ActionEvent evt )
    {// GEN-FIRST:event_btnOvNaActionPerformed
        final XmlOverlongNames xmlOverlongNames = (XmlOverlongNames) mapBtn.get( evt.getSource() );
        ConfigureXmlOverlongNamesDialog_NB dlg = new ConfigureXmlOverlongNamesDialog_NB( xmlOverlongNames );
        dlg.setTitle( xmlOverlongNames.getName() );
        dlg.setLocationRelativeTo( this );

        if ( dlg.showDialog() )
        {
            logger.trace( "OK pressed" );
        }
    }// GEN-LAST:event_btnOvNaActionPerformed

    private void btnEnExActionPerformed( java.awt.event.ActionEvent evt )
    {// GEN-FIRST:event_btnEnExActionPerformed
        final XmlEntityExpansion xmlEntityExpansion = (XmlEntityExpansion) mapBtn.get( evt.getSource() );
        ConfigureXmlEntityExpansionDialog_NB dlg = new ConfigureXmlEntityExpansionDialog_NB( xmlEntityExpansion );
        dlg.setTitle( xmlEntityExpansion.getName() );
        dlg.setLocationRelativeTo( this );

        if ( dlg.showDialog() )
        {
            logger.trace( "OK pressed" );
        }
    }// GEN-LAST:event_btnEnExActionPerformed

    private void btnExEnActionPerformed( java.awt.event.ActionEvent evt )
    {// GEN-FIRST:event_btnExEnActionPerformed
        final XmlExternalEntity xmlExternalEntity = (XmlExternalEntity) mapBtn.get( evt.getSource() );
        ConfigureXmlExternalEntityDialog_NB dlg = new ConfigureXmlExternalEntityDialog_NB( xmlExternalEntity );
        dlg.setTitle( xmlExternalEntity.getName() );
        dlg.setLocationRelativeTo( this );

        if ( dlg.showDialog() )
        {
            logger.trace( "OK pressed" );
        }
    }// GEN-LAST:event_btnExEnActionPerformed

    private void btnHaCoActionPerformed( java.awt.event.ActionEvent evt )
    {// GEN-FIRST:event_btnHaCoActionPerformed
        final HashCollision hashCollision = (HashCollision) mapBtn.get( evt.getSource() );
        ConfigureHashCollisionDialog_NB dlg = new ConfigureHashCollisionDialog_NB( hashCollision );
        dlg.setTitle( hashCollision.getName() );
        dlg.setLocationRelativeTo( this );

        if ( dlg.showDialog() )
        {
            logger.trace( "OK pressed" );
        }
    }// GEN-LAST:event_btnHaCoActionPerformed

    private void btnAtCoActionPerformed( java.awt.event.ActionEvent evt )
    {// GEN-FIRST:event_btnAtCoActionPerformed
        final XmlAttributeCount xmlAttributeCount = (XmlAttributeCount) mapBtn.get( evt.getSource() );
        ConfigureXmlAttributeCountDialog_NB dlg = new ConfigureXmlAttributeCountDialog_NB( xmlAttributeCount );
        dlg.setTitle( xmlAttributeCount.getName() );
        dlg.setLocationRelativeTo( this );

        if ( dlg.showDialog() )
        {
            logger.trace( "OK pressed" );
        }
    }// GEN-LAST:event_btnAtCoActionPerformed

    private void btnCoPaActionPerformed( java.awt.event.ActionEvent evt )
    {// GEN-FIRST:event_btnCoPaActionPerformed
     // GEN-FIRST:event_jButton1ActionPerformed
        final CoerciveParsing coerciveParsing = (CoerciveParsing) mapBtn.get( evt.getSource() );
        ConfigureCoerceiveParsingDialog_NB dlg = new ConfigureCoerceiveParsingDialog_NB( coerciveParsing );
        dlg.setTitle( coerciveParsing.getName() );
        dlg.setLocationRelativeTo( this );

        if ( dlg.showDialog() )
        {
            logger.trace( "OK pressed" );
        }
    }// GEN-LAST:event_btnCoPaActionPerformed

    private void btnElCoActionPerformed( java.awt.event.ActionEvent evt )
    {// GEN-FIRST:event_btnElCoActionPerformed
        final XmlElementCount xmlElementCount = (XmlElementCount) mapBtn.get( evt.getSource() );
        ConfigureXmlElementCountDialog_NB dlg = new ConfigureXmlElementCountDialog_NB( xmlElementCount );
        dlg.setTitle( xmlElementCount.getName() );
        dlg.setLocationRelativeTo( this );

        if ( dlg.showDialog() )
        {
            logger.trace( "OK pressed" );
        }
    }// GEN-LAST:event_btnElCoActionPerformed

    private void cbCoPaActionPerformed( java.awt.event.ActionEvent evt )
    {// GEN-FIRST:event_cbCoPaActionPerformed
        btnCoPa.setEnabled( cbCoPa.isSelected() );
    }// GEN-LAST:event_cbCoPaActionPerformed

    private void cbElCoActionPerformed( java.awt.event.ActionEvent evt )
    {// GEN-FIRST:event_cbElCoActionPerformed
        btnElCo.setEnabled( cbElCo.isSelected() );
    }// GEN-LAST:event_cbElCoActionPerformed

    private void cbAtCoActionPerformed( java.awt.event.ActionEvent evt )
    {// GEN-FIRST:event_cbAtCoActionPerformed
        btnAtCo.setEnabled( cbAtCo.isSelected() );
    }// GEN-LAST:event_cbAtCoActionPerformed

    private void cbEnExActionPerformed( java.awt.event.ActionEvent evt )
    {// GEN-FIRST:event_cbEnExActionPerformed
        btnEnEx.setEnabled( cbEnEx.isSelected() );
    }// GEN-LAST:event_cbEnExActionPerformed

    private void cbExEnActionPerformed( java.awt.event.ActionEvent evt )
    {// GEN-FIRST:event_cbExEnActionPerformed
        btnExEn.setEnabled( cbExEn.isSelected() );
    }// GEN-LAST:event_cbExEnActionPerformed

    private void cbHaCoActionPerformed( java.awt.event.ActionEvent evt )
    {// GEN-FIRST:event_cbHaCoActionPerformed
        btnHaCo.setEnabled( cbHaCo.isSelected() );
    }// GEN-LAST:event_cbHaCoActionPerformed

    private void cbOvNaActionPerformed( java.awt.event.ActionEvent evt )
    {// GEN-FIRST:event_cbOvNaActionPerformed
        btnOvNa.setEnabled( cbOvNa.isSelected() );
    }// GEN-LAST:event_cbOvNaActionPerformed

    private void btnOkActionPerformed( java.awt.event.ActionEvent evt )
    {// GEN-FIRST:event_jButton8ActionPerformed
        setVisible( false );

        ok = true;
        dispose();
    }// GEN-LAST:event_jButton8ActionPerformed

    public DoSAttack[] getAttacks()
    {
        List<DoSAttack> list = new ArrayList<DoSAttack>();
        for ( Entry<JCheckBox, DoSAttack> entry : map.entrySet() )
        {
            if ( entry.getKey().isSelected() )
            {
                list.add( entry.getValue() );
            }
        }

        DoSAttack[] array = new DoSAttack[list.size()];

        return list.toArray( array );
    }

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton btnAtCo;

    private javax.swing.JButton btnCoPa;

    private javax.swing.JButton btnElCo;

    private javax.swing.JButton btnEnEx;

    private javax.swing.JButton btnExEn;

    private javax.swing.JButton btnHaCo;

    private javax.swing.JButton btnOk;

    private javax.swing.JButton btnOvNa;

    private javax.swing.JCheckBox cbAtCo;

    private javax.swing.JCheckBox cbCoPa;

    private javax.swing.JCheckBox cbElCo;

    private javax.swing.JCheckBox cbEnEx;

    private javax.swing.JCheckBox cbExEn;

    private javax.swing.JCheckBox cbHaCo;

    private javax.swing.JCheckBox cbOvNa;

    // End of variables declaration//GEN-END:variables
    public boolean showDialog()
    {
        // set the dialog to modal and display it
        setModal( true );
        setVisible( true );

        return ok;
    }
}
