/**
 * WS-Attacker - A Modular Web Services Penetration Testing Framework Copyright
 * (C) 2013 Christian Altmeier
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */
package wsattacker.plugin.intelligentdos.ui.dos;

import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import wsattacker.library.intelligentdos.dos.HashCollision;
import wsattacker.library.intelligentdos.hashdos.CollisionDJBX;
import wsattacker.library.intelligentdos.helper.IterateModel;
import wsattacker.plugin.dos.dosExtension.attackClasses.hashDos.CollisionDJBX31A;
import wsattacker.plugin.dos.dosExtension.attackClasses.hashDos.CollisionDJBX33A;
import wsattacker.plugin.dos.dosExtension.attackClasses.hashDos.CollisionDJBX33X;
import wsattacker.plugin.dos.dosExtension.attackClasses.hashDos.CollisionInterface;

/**
 * @author Christian Altmeier
 */
public class ConfigureHashCollisionDialog_NB
    extends javax.swing.JDialog
{

    private transient final HashCollision hashCollision;

    private final CBChangeListener cbChangeListener = new CBChangeListener();

    private boolean ok;

    /**
     * Creates new form ConfigureCoerceiveParsingDialog_NB
     * 
     * @param hashCollision
     */
    public ConfigureHashCollisionDialog_NB( HashCollision hashCollision )
    {
        this.hashCollision = hashCollision;
        initComponents();

        cbDJBX31A.addChangeListener( cbChangeListener );
        cbDJBX33A.addChangeListener( cbChangeListener );
        cbDJBX33X.addChangeListener( cbChangeListener );
        cbRuby.addChangeListener( cbChangeListener );
        cbTrue.addChangeListener( cbChangeListener );
        cbFalse.addChangeListener( cbChangeListener );

        final CollisionInterface[] collisionGenerators = hashCollision.getCollisionGenerators();
        for ( CollisionInterface collision : collisionGenerators )
        {
            if ( collision instanceof CollisionDJBX31A )
            {
                cbDJBX31A.setSelected( true );
            }
            if ( collision instanceof CollisionDJBX33A )
            {
                cbDJBX33A.setSelected( true );
            }
            if ( collision instanceof CollisionDJBX33X )
            {
                cbDJBX33X.setSelected( true );
            }
            if ( collision instanceof CollisionDJBX )
            {
                cbRuby.setSelected( true );
            }
        }

        final Boolean[] useNamespace = hashCollision.getUseNamespace();
        for ( Boolean b : useNamespace )
        {
            if ( b )
            {
                cbTrue.setSelected( true );
            }

            if ( !b )
            {
                cbFalse.setSelected( true );
            }
        }

        final IterateModel numberOfAttributesIterator = hashCollision.getNumberOfCollisionsIterator();
        configureIterateModel_NB2.setStartAt( String.valueOf( numberOfAttributesIterator.getStartAt() ) );
        configureIterateModel_NB2.setStopAt( String.valueOf( numberOfAttributesIterator.getStopAt() ) );
        configureIterateModel_NB2.setIncrement( String.valueOf( numberOfAttributesIterator.getIncrement() ) );
        configureIterateModel_NB2.setIterateStrategie( numberOfAttributesIterator.getIterateStrategie() );
    }

    /**
     * This method is called from within the constructor to initialize the form. WARNING: Do NOT modify this code. The
     * content of this method is always regenerated by the Form Editor.
     */
    @SuppressWarnings( "unchecked" )
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents()
    {

        jLabel1 = new javax.swing.JLabel();
        jSeparator1 = new javax.swing.JSeparator();
        jButton1 = new javax.swing.JButton();
        jSeparator3 = new javax.swing.JSeparator();
        configureIterateModel_NB2 = new wsattacker.plugin.intelligentdos.ui.components.ConfigureIterateModel_NB();
        cbDJBX31A = new javax.swing.JCheckBox();
        cbDJBX33A = new javax.swing.JCheckBox();
        cbDJBX33X = new javax.swing.JCheckBox();
        cbRuby = new javax.swing.JCheckBox();
        jLabel2 = new javax.swing.JLabel();
        jSeparator2 = new javax.swing.JSeparator();
        cbTrue = new javax.swing.JCheckBox();
        jLabel3 = new javax.swing.JLabel();
        cbFalse = new javax.swing.JCheckBox();

        setDefaultCloseOperation( javax.swing.WindowConstants.DISPOSE_ON_CLOSE );

        jLabel1.setText( "Number of attributes" );

        jButton1.setText( "OK" );
        jButton1.setEnabled( isOkEnabled() );
        jButton1.addActionListener( new java.awt.event.ActionListener()
        {
            @Override
            public void actionPerformed( java.awt.event.ActionEvent evt )
            {
                jButton1ActionPerformed( evt );
            }
        } );

        cbDJBX31A.setText( "DJBX31A" );

        cbDJBX33A.setText( "DJBX33A" );

        cbDJBX33X.setText( "DJBX33X" );

        cbRuby.setText( "Ruby" );

        jLabel2.setText( "Hash implementation" );

        cbTrue.setText( "True" );

        jLabel3.setText( "use namespace" );

        cbFalse.setText( "False" );

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout( getContentPane() );
        getContentPane().setLayout( layout );
        layout.setHorizontalGroup( layout.createParallelGroup( javax.swing.GroupLayout.Alignment.LEADING ).addGroup( layout.createSequentialGroup().addContainerGap().addGroup( layout.createParallelGroup( javax.swing.GroupLayout.Alignment.LEADING ).addComponent( cbRuby ).addComponent( jSeparator2 ).addComponent( jLabel2 ).addGroup( layout.createSequentialGroup().addGroup( layout.createParallelGroup( javax.swing.GroupLayout.Alignment.LEADING ).addComponent( cbDJBX31A ).addComponent( cbDJBX33A ).addComponent( cbDJBX33X ) ).addGap( 47,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      47,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      47 ).addGroup( layout.createParallelGroup( javax.swing.GroupLayout.Alignment.LEADING ).addComponent( jLabel3 ).addGroup( layout.createSequentialGroup().addGap( 10,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      10,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      10 ).addGroup( layout.createParallelGroup( javax.swing.GroupLayout.Alignment.LEADING ).addComponent( cbTrue ).addComponent( cbFalse ) ) ) ) ).addComponent( jSeparator1 ).addComponent( jSeparator3 ).addComponent( jLabel1 ).addComponent( jButton1 ).addComponent( configureIterateModel_NB2,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                           javax.swing.GroupLayout.PREFERRED_SIZE,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                           javax.swing.GroupLayout.DEFAULT_SIZE,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                           javax.swing.GroupLayout.PREFERRED_SIZE ) ).addContainerGap() ) );
        layout.setVerticalGroup( layout.createParallelGroup( javax.swing.GroupLayout.Alignment.LEADING ).addGroup( javax.swing.GroupLayout.Alignment.TRAILING,
                                                                                                                   layout.createSequentialGroup().addContainerGap().addComponent( jLabel2 ).addPreferredGap( javax.swing.LayoutStyle.ComponentPlacement.RELATED ).addComponent( jSeparator2,
                                                                                                                                                                                                                                                                                javax.swing.GroupLayout.PREFERRED_SIZE,
                                                                                                                                                                                                                                                                                10,
                                                                                                                                                                                                                                                                                javax.swing.GroupLayout.PREFERRED_SIZE ).addPreferredGap( javax.swing.LayoutStyle.ComponentPlacement.RELATED ).addGroup( layout.createParallelGroup( javax.swing.GroupLayout.Alignment.BASELINE ).addComponent( cbDJBX31A ).addComponent( jLabel3 ) ).addPreferredGap( javax.swing.LayoutStyle.ComponentPlacement.RELATED ).addGroup( layout.createParallelGroup( javax.swing.GroupLayout.Alignment.BASELINE ).addComponent( cbDJBX33A ).addComponent( cbFalse ) ).addPreferredGap( javax.swing.LayoutStyle.ComponentPlacement.RELATED ).addGroup( layout.createParallelGroup( javax.swing.GroupLayout.Alignment.BASELINE ).addComponent( cbDJBX33X ).addComponent( cbTrue ) ).addPreferredGap( javax.swing.LayoutStyle.ComponentPlacement.RELATED ).addComponent( cbRuby ).addPreferredGap( javax.swing.LayoutStyle.ComponentPlacement.UNRELATED ).addComponent( jLabel1 ).addPreferredGap( javax.swing.LayoutStyle.ComponentPlacement.RELATED ).addComponent( jSeparator1,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                javax.swing.GroupLayout.PREFERRED_SIZE,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                10,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                javax.swing.GroupLayout.PREFERRED_SIZE ).addPreferredGap( javax.swing.LayoutStyle.ComponentPlacement.RELATED ).addComponent( configureIterateModel_NB2,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                             javax.swing.GroupLayout.PREFERRED_SIZE,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                             javax.swing.GroupLayout.DEFAULT_SIZE,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                             javax.swing.GroupLayout.PREFERRED_SIZE ).addPreferredGap( javax.swing.LayoutStyle.ComponentPlacement.RELATED ).addComponent( jSeparator3,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          javax.swing.GroupLayout.PREFERRED_SIZE,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          10,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          javax.swing.GroupLayout.PREFERRED_SIZE ).addPreferredGap( javax.swing.LayoutStyle.ComponentPlacement.RELATED ).addComponent( jButton1 ).addContainerGap( javax.swing.GroupLayout.DEFAULT_SIZE,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                   Short.MAX_VALUE ) ) );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void jButton1ActionPerformed( java.awt.event.ActionEvent evt )
    {// GEN-FIRST:event_jButton1ActionPerformed
        setVisible( false );
        dispose();

        setCollisionGenerators();
        setUseNamespace();
        setIterateModel();

        ok = true;
    }// GEN-LAST:event_jButton1ActionPerformed

    private void setCollisionGenerators()
    {
        List<CollisionInterface> tmp = new ArrayList<CollisionInterface>();
        if ( cbDJBX31A.isSelected() )
        {
            tmp.add( new CollisionDJBX31A() );
        }
        if ( cbDJBX33A.isSelected() )
        {
            tmp.add( new CollisionDJBX33A() );
        }
        if ( cbDJBX33X.isSelected() )
        {
            tmp.add( new CollisionDJBX33X() );
        }
        if ( cbRuby.isSelected() )
        {
            tmp.add( new CollisionDJBX() );
        }

        if ( tmp.isEmpty() )
        {
            throw new IllegalStateException( "no checkbox for collision checked!" );
        }

        CollisionInterface[] a = new CollisionInterface[tmp.size()];
        hashCollision.setCollisionGenerators( tmp.toArray( a ) );
    }

    private void setUseNamespace()
    {
        if ( cbTrue.isSelected() && cbFalse.isSelected() )
        {
            Boolean[] useNamespace = { Boolean.FALSE, Boolean.TRUE };
            hashCollision.setUseNamespace( useNamespace );
        }
        else if ( cbTrue.isSelected() )
        {
            Boolean[] useNamespace = { Boolean.TRUE };
            hashCollision.setUseNamespace( useNamespace );
        }
        else if ( cbFalse.isSelected() )
        {
            Boolean[] useNamespace = { Boolean.FALSE };
            hashCollision.setUseNamespace( useNamespace );
        }
        else
        {
            throw new IllegalStateException( "no checkbox for useNamespace checked!" );
        }
    }

    private void setIterateModel()
    {
        int startAt = Integer.parseInt( configureIterateModel_NB2.getStartAt() );
        int stopAt = Integer.parseInt( configureIterateModel_NB2.getStopAt() );
        int increment = Integer.parseInt( configureIterateModel_NB2.getIncrement() );

        final IterateModel iterateModel =
            IterateModel.custom().startAt( startAt ).stopAt( stopAt ).setIncrement( increment ).setIterateStrategie( configureIterateModel_NB2.getIterateStrategie() ).build();

        hashCollision.setNumberOfCollisionsIterator( iterateModel );
    }

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JCheckBox cbDJBX31A;

    private javax.swing.JCheckBox cbDJBX33A;

    private javax.swing.JCheckBox cbDJBX33X;

    private javax.swing.JCheckBox cbFalse;

    private javax.swing.JCheckBox cbRuby;

    private javax.swing.JCheckBox cbTrue;

    private wsattacker.plugin.intelligentdos.ui.components.ConfigureIterateModel_NB configureIterateModel_NB2;

    private javax.swing.JButton jButton1;

    private javax.swing.JLabel jLabel1;

    private javax.swing.JLabel jLabel2;

    private javax.swing.JLabel jLabel3;

    private javax.swing.JSeparator jSeparator1;

    private javax.swing.JSeparator jSeparator2;

    private javax.swing.JSeparator jSeparator3;

    // End of variables declaration//GEN-END:variables

    public boolean showDialog()
    {
        setModal( true );
        setVisible( true );

        addWindowFocusListener( new WindowAdapter()
        {

            @Override
            public void windowClosing( WindowEvent e )
            {
                ok = false;
            }
        } );

        return ok;
    }

    private boolean isOkEnabled()
    {
        return ( cbDJBX31A.isSelected() || cbDJBX33A.isSelected() || cbDJBX33X.isSelected() || cbRuby.isSelected() )
            && ( cbTrue.isSelected() || cbFalse.isSelected() );
    }

    private class CBChangeListener
        implements ChangeListener, Serializable
    {

        /**
		 * 
		 */
        private static final long serialVersionUID = 1L;

        @Override
        public void stateChanged( ChangeEvent e )
        {
            jButton1.setEnabled( isOkEnabled() );
        }

    }

}
