/**
 * WS-Attacker - A Modular Web Services Penetration Testing Framework Copyright
 * (C) 2013 Christian Altmeier
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */
package wsattacker.plugin.intelligentdos.ui.option;

import java.util.ArrayList;
import java.util.List;
import wsattacker.gui.component.pluginconfiguration.composition.OptionGUI;
import wsattacker.library.intelligentdos.dos.DoSAttack;
import wsattacker.library.intelligentdos.helper.CommonParamItem;
import wsattacker.main.composition.plugin.option.AbstractOption;
import wsattacker.plugin.intelligentdos.ui.dialog.ConfigureAttacksDialog_NB;
import wsattacker.plugin.intelligentdos.ui.dialog.ConfigureCommonParams_NB;
import wsattacker.plugin.intelligentdos.ui.helper.JTextFieldInteger;

/**
 * @author Christian Altmeier
 */
public class ConfigureAttacksOptionGUI_NB
    extends OptionGUI
{

    /**
	 * 
	 */
    private static final long serialVersionUID = 1L;

    private int serverRecoveryTime;

    private int httpConnectionTimeout;

    private transient DoSAttack[] attacks;

    private List<CommonParamItem> commonParamList;

    public ConfigureAttacksOptionGUI_NB( DoSAttack[] attacks, List<CommonParamItem> commonParamList,
                                         int serverRecoveryTime, int httpConnectionTimeout )
    {
        this.attacks = new DoSAttack[attacks.length];
        System.arraycopy( attacks, 0, this.attacks, 0, attacks.length );
        this.serverRecoveryTime = serverRecoveryTime;
        this.httpConnectionTimeout = httpConnectionTimeout;
        if ( commonParamList != null )
        {
            this.commonParamList = commonParamList;
        }
        else
        {
            this.commonParamList = new ArrayList<CommonParamItem>();
        }

        initComponents();

        serverRecoveryTextField.setDocument( new JTextFieldInteger() );
        connectionTimeoutTextField.setDocument( new JTextFieldInteger() );

        serverRecoveryTextField.setText( String.valueOf( serverRecoveryTime ) );
        connectionTimeoutTextField.setText( String.valueOf( httpConnectionTimeout ) );
    }

    /**
     * This method is called from within the constructor to initialize the form. WARNING: Do NOT modify this code. The
     * content of this method is always regenerated by the Form Editor.
     */
    @SuppressWarnings( "unchecked" )
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents()
    {
        bindingGroup = new org.jdesktop.beansbinding.BindingGroup();

        jLabel1 = new javax.swing.JLabel();
        configureAttacks = new javax.swing.JButton();
        jLabel2 = new javax.swing.JLabel();
        btnConfigureCommonParams = new javax.swing.JButton();
        jLabel3 = new javax.swing.JLabel();
        serverRecoveryTextField = new javax.swing.JTextField();
        jLabel4 = new javax.swing.JLabel();
        connectionTimeoutTextField = new javax.swing.JTextField();

        jLabel1.setText( "Configure DoS Attacks" );

        configureAttacks.setText( "configure" );
        configureAttacks.addActionListener( new java.awt.event.ActionListener()
        {
            @Override
            public void actionPerformed( java.awt.event.ActionEvent evt )
            {
                configureAttacksActionPerformed( evt );
            }
        } );

        jLabel2.setText( "Configure common parameter" );

        btnConfigureCommonParams.setText( "configure" );
        btnConfigureCommonParams.addActionListener( new java.awt.event.ActionListener()
        {
            @Override
            public void actionPerformed( java.awt.event.ActionEvent evt )
            {
                btnConfigureCommonParamsActionPerformed( evt );
            }
        } );

        jLabel3.setText( "server recovery time (millies)" );

        serverRecoveryTextField.setHorizontalAlignment( javax.swing.JTextField.RIGHT );

        org.jdesktop.beansbinding.Binding binding =
            org.jdesktop.beansbinding.Bindings.createAutoBinding( org.jdesktop.beansbinding.AutoBinding.UpdateStrategy.READ_WRITE,
                                                                  this,
                                                                  org.jdesktop.beansbinding.ELProperty.create( "${serverRecoveryTime}" ),
                                                                  serverRecoveryTextField,
                                                                  org.jdesktop.beansbinding.BeanProperty.create( "text" ) );
        bindingGroup.addBinding( binding );

        jLabel4.setText( "HTTP connection timeout (millies)" );

        connectionTimeoutTextField.setHorizontalAlignment( javax.swing.JTextField.RIGHT );

        binding =
            org.jdesktop.beansbinding.Bindings.createAutoBinding( org.jdesktop.beansbinding.AutoBinding.UpdateStrategy.READ_WRITE,
                                                                  this,
                                                                  org.jdesktop.beansbinding.ELProperty.create( "${httpConnectionTimeout}" ),
                                                                  connectionTimeoutTextField,
                                                                  org.jdesktop.beansbinding.BeanProperty.create( "text" ) );
        bindingGroup.addBinding( binding );

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout( this );
        this.setLayout( layout );
        layout.setHorizontalGroup( layout.createParallelGroup( javax.swing.GroupLayout.Alignment.LEADING ).addGroup( layout.createSequentialGroup().addContainerGap().addGroup( layout.createParallelGroup( javax.swing.GroupLayout.Alignment.LEADING ).addGroup( layout.createSequentialGroup().addGroup( layout.createParallelGroup( javax.swing.GroupLayout.Alignment.LEADING ).addComponent( jLabel3 ).addComponent( jLabel4 ) ).addPreferredGap( javax.swing.LayoutStyle.ComponentPlacement.RELATED,
                                                                                                                                                                                                                                                                                                                                                                                                                                                      65,
                                                                                                                                                                                                                                                                                                                                                                                                                                                      Short.MAX_VALUE ).addGroup( layout.createParallelGroup( javax.swing.GroupLayout.Alignment.LEADING ).addComponent( serverRecoveryTextField,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        javax.swing.GroupLayout.Alignment.TRAILING,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        javax.swing.GroupLayout.PREFERRED_SIZE,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        77,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        javax.swing.GroupLayout.PREFERRED_SIZE ).addComponent( connectionTimeoutTextField,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                               javax.swing.GroupLayout.Alignment.TRAILING,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                               javax.swing.GroupLayout.PREFERRED_SIZE,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                               77,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                               javax.swing.GroupLayout.PREFERRED_SIZE ) ) ).addGroup( layout.createSequentialGroup().addComponent( jLabel1 ).addPreferredGap( javax.swing.LayoutStyle.ComponentPlacement.RELATED,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                              javax.swing.GroupLayout.DEFAULT_SIZE,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                              Short.MAX_VALUE ).addComponent( configureAttacks ) ).addGroup( layout.createSequentialGroup().addComponent( jLabel2 ).addPreferredGap( javax.swing.LayoutStyle.ComponentPlacement.RELATED,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     javax.swing.GroupLayout.DEFAULT_SIZE,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     Short.MAX_VALUE ).addComponent( btnConfigureCommonParams ) ) ).addContainerGap() ) );
        layout.setVerticalGroup( layout.createParallelGroup( javax.swing.GroupLayout.Alignment.LEADING ).addGroup( layout.createSequentialGroup().addContainerGap().addGroup( layout.createParallelGroup( javax.swing.GroupLayout.Alignment.BASELINE ).addComponent( jLabel1 ).addComponent( configureAttacks ) ).addPreferredGap( javax.swing.LayoutStyle.ComponentPlacement.RELATED ).addGroup( layout.createParallelGroup( javax.swing.GroupLayout.Alignment.BASELINE ).addComponent( jLabel2 ).addComponent( btnConfigureCommonParams ) ).addPreferredGap( javax.swing.LayoutStyle.ComponentPlacement.RELATED ).addGroup( layout.createParallelGroup( javax.swing.GroupLayout.Alignment.BASELINE ).addComponent( jLabel3 ).addComponent( serverRecoveryTextField,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                             javax.swing.GroupLayout.PREFERRED_SIZE,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                             javax.swing.GroupLayout.DEFAULT_SIZE,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                             javax.swing.GroupLayout.PREFERRED_SIZE ) ).addPreferredGap( javax.swing.LayoutStyle.ComponentPlacement.RELATED ).addGroup( layout.createParallelGroup( javax.swing.GroupLayout.Alignment.BASELINE ).addComponent( jLabel4 ).addComponent( connectionTimeoutTextField,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       javax.swing.GroupLayout.PREFERRED_SIZE,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       javax.swing.GroupLayout.DEFAULT_SIZE,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       javax.swing.GroupLayout.PREFERRED_SIZE ) ).addContainerGap( javax.swing.GroupLayout.DEFAULT_SIZE,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                   Short.MAX_VALUE ) ) );

        bindingGroup.bind();
    }// </editor-fold>//GEN-END:initComponents

    // dos attacks
    private void configureAttacksActionPerformed( java.awt.event.ActionEvent evt )
    {
        // create a new jdialog for configuring the attacks
        ConfigureAttacksDialog_NB configureDialog = new ConfigureAttacksDialog_NB( attacks );
        configureDialog.setLocationRelativeTo( this );
        if ( configureDialog.showDialog() )
        {
            attacks = configureDialog.getAttacks();
        }
    }

    // common params
    private void btnConfigureCommonParamsActionPerformed( java.awt.event.ActionEvent evt )
    {// GEN-FIRST:event_jButton2ActionPerformed
        ConfigureCommonParams_NB configureCommonParams = new ConfigureCommonParams_NB( commonParamList );
        configureCommonParams.setLocationRelativeTo( this );
        if ( configureCommonParams.showDialog() )
        {
            commonParamList = configureCommonParams.getCommonParams();
        }

    }// GEN-LAST:event_jButton2ActionPerformed

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton btnConfigureCommonParams;

    private javax.swing.JButton configureAttacks;

    private javax.swing.JTextField connectionTimeoutTextField;

    private javax.swing.JLabel jLabel1;

    private javax.swing.JLabel jLabel2;

    private javax.swing.JLabel jLabel3;

    private javax.swing.JLabel jLabel4;

    private javax.swing.JTextField serverRecoveryTextField;

    private transient org.jdesktop.beansbinding.BindingGroup bindingGroup;

    // End of variables declaration//GEN-END:variables

    public DoSAttack[] getAttacks()
    {
        DoSAttack[] copy = new DoSAttack[attacks.length];
        System.arraycopy( attacks, 0, copy, 0, attacks.length );
        return copy;
    }

    public List<CommonParamItem> getCommonParamList()
    {
        return commonParamList;
    }

    public int getServerRecoveryTime()
    {
        return serverRecoveryTime;
    }

    public void setServerRecoveryTime( int serverRecoveryTime )
    {
        this.serverRecoveryTime = serverRecoveryTime;
    }

    public int getHttpConnectionTimeout()
    {
        return httpConnectionTimeout;
    }

    public void setHttpConnectionTimeout( int httpConnectionTimeout )
    {
        this.httpConnectionTimeout = httpConnectionTimeout;
    }

    @Override
    public void bindingDoUnbind()
    {

    }

    @Override
    public AbstractOption getUsedOption()
    {
        return null;
    }
}
