/**
 * WS-Attacker - A Modular Web Services Penetration Testing Framework Copyright
 * (C) 2013 Dennis Kupser
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */
package wsattacker.plugin.xmlencryptionattack.serverbehaviour.gui;

import java.awt.Component;
import java.awt.HeadlessException;
import java.io.File;
import java.io.IOException;
import java.security.NoSuchAlgorithmException;
import java.security.NoSuchProviderException;
import java.security.cert.CertificateException;
import java.security.spec.InvalidKeySpecException;
import java.util.List;
import javax.swing.DefaultCellEditor;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.filechooser.FileFilter;
import javax.swing.filechooser.FileNameExtensionFilter;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableColumn;
import javax.xml.bind.JAXBException;
import org.apache.log4j.Logger;
import wsattacker.gui.component.pluginconfiguration.composition.OptionGUI;
import wsattacker.library.xmlencryptionattack.attackengine.CryptoAttackException;
import wsattacker.library.xmlencryptionattack.attackengine.oracle.base.response.OracleResponse;
import wsattacker.library.xmlencryptionattack.attackengine.oracle.base.response.OracleResponse.Result;
import wsattacker.library.xmlencryptionattack.encryptedelements.data.EncryptedDataElement;
import wsattacker.library.xmlencryptionattack.encryptedelements.key.EncryptedKeyElement;
import wsattacker.library.xmlencryptionattack.util.XMLEncryptionConstants.XMLEncryptionAttackMode;
import wsattacker.main.composition.plugin.AbstractPlugin;
import wsattacker.main.composition.plugin.option.AbstractOption;
import wsattacker.plugin.xmlencryptionattack.XMLEncryptionAttack;
import wsattacker.plugin.xmlencryptionattack.option.OptionServerErrorBehaviour;
import static wsattacker.plugin.xmlencryptionattack.serverbehaviour.gui.OracleResponseTableModel.COLUMN_IDX_ORACLERESULT;
import static wsattacker.plugin.xmlencryptionattack.serverbehaviour.gui.OracleResponseTableModel.COLUMN_IDX_ROW;

/**
 * @author Dennis Kupser
 */
public class ServerBehaviourGUI
    extends OptionGUI
{

    private static final Logger LOG = Logger.getLogger( ServerBehaviourGUI.class );

    private final OptionServerErrorBehaviour m_Option;

    private final XMLEncryptionAttack m_EncryptionPlugin;

    /**
     * Creates new form ServerBehaviourGUI
     * 
     * @param plugin
     * @param option
     */
    public ServerBehaviourGUI( AbstractPlugin plugin, OptionServerErrorBehaviour option )
    {
        this.m_EncryptionPlugin = (XMLEncryptionAttack) plugin;
        this.m_Option = option;
        initComponents();
        initTable();
        cbPKCS1GenWithEncData.setEnabled( false );
    }

    /**
     * This method is called from within the constructor to initialize the form. WARNING: Do NOT modify this code. The
     * content of this method is always regenerated by the Form Editor.
     */
    @SuppressWarnings( "unchecked" )
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents()
    {

        chooseLoad = new javax.swing.JFileChooser();
        chooseSaveFile = new javax.swing.JFileChooser();
        chooseLoadCert = new javax.swing.JFileChooser();
        chooseLoadPubKey = new javax.swing.JFileChooser();
        jScrollPane1 = new javax.swing.JScrollPane();
        tableOracleResponse = new javax.swing.JTable();
        tableOracleResponse.getSelectionModel().addListSelectionListener( new ListSelectionListener()
        {

            @Override
            public void valueChanged( ListSelectionEvent e )
            {
                int selectedRow = tableOracleResponse.getSelectedRow();
                final List<OracleResponse> responseData = m_Option.getResponseData();
                if ( selectedRow >= 0 && selectedRow < responseData.size() )
                {
                    String response = responseData.get( selectedRow ).getResponse();
                    String request = responseData.get( selectedRow ).getRequest();
                    requestViewer.setText( request );
                    responseViewer.setText( response );
                }
                else
                {
                    requestViewer.setText( "" );
                    responseViewer.setText( "" );
                }
            }
        } );
        btnLoadFile = new javax.swing.JButton();
        btnSaveFile = new javax.swing.JButton();
        btnNewMsgs = new javax.swing.JButton();
        btnLoadServerKey = new javax.swing.JButton();
        ServErrMsgProgressBar = new javax.swing.JProgressBar();
        lbServerError = new javax.swing.JLabel();
        lbTableRows = new javax.swing.JLabel();
        tbRowCount = new javax.swing.JTextField();
        btnAbort = new javax.swing.JButton();
        btnLoadPubKey = new javax.swing.JButton();
        cbPKCS1GenWithEncData = new javax.swing.JCheckBox();
        cbIgnorePayResponse = new javax.swing.JCheckBox();
        requestResponseSplitPane = new javax.swing.JSplitPane();
        rTextScrollPane1 = new org.fife.ui.rtextarea.RTextScrollPane();
        requestViewer = new org.fife.ui.rsyntaxtextarea.RSyntaxTextArea();
        requestViewer.setLineWrap( true );
        rTextScrollPane2 = new org.fife.ui.rtextarea.RTextScrollPane();
        responseViewer = new org.fife.ui.rsyntaxtextarea.RSyntaxTextArea();
        responseViewer.setLineWrap( true );

        chooseLoad.setDialogTitle( "Load Server Responses" );
        FileFilter loadfilter = new FileNameExtensionFilter( "XML File", "xml" );
        chooseLoad.addChoosableFileFilter( loadfilter );
        chooseLoad.setFileFilter( loadfilter );
        chooseLoad.setFileHidingEnabled( false );
        chooseLoad.addActionListener( new java.awt.event.ActionListener()
        {
            public void actionPerformed( java.awt.event.ActionEvent evt )
            {
                chooseLoadActionPerformed( evt );
            }
        } );

        chooseSaveFile.setDialogType( javax.swing.JFileChooser.SAVE_DIALOG );
        chooseSaveFile.setDialogTitle( "Save File" );
        FileFilter saveFilter = new FileNameExtensionFilter( "XML File", "xml" );
        chooseSaveFile.addChoosableFileFilter( saveFilter );
        chooseSaveFile.setFileFilter( saveFilter );
        chooseSaveFile.addActionListener( new java.awt.event.ActionListener()
        {
            public void actionPerformed( java.awt.event.ActionEvent evt )
            {
                chooseSaveFileActionPerformed( evt );
            }
        } );

        chooseLoadCert.setDialogTitle( "Load Server Certificate" );
        // FileFilter loadfilter = new FileNameExtensionFilter("XML File","xml");
        // LoadCertChooser.addChoosableFileFilter(loadfilter);
        // LoadCertChooser.setFileFilter(loadfilter);
        chooseLoadCert.setFileHidingEnabled( false );
        chooseLoadCert.addActionListener( new java.awt.event.ActionListener()
        {
            public void actionPerformed( java.awt.event.ActionEvent evt )
            {
                chooseLoadCertActionPerformed( evt );
            }
        } );

        chooseLoadPubKey.addActionListener( new java.awt.event.ActionListener()
        {
            public void actionPerformed( java.awt.event.ActionEvent evt )
            {
                chooseLoadPubKeyActionPerformed( evt );
            }
        } );

        tableOracleResponse.setAutoResizeMode( javax.swing.JTable.AUTO_RESIZE_ALL_COLUMNS );
        tableOracleResponse.setCellSelectionEnabled( true );
        tableOracleResponse.setSelectionMode( javax.swing.ListSelectionModel.SINGLE_SELECTION );
        jScrollPane1.setViewportView( tableOracleResponse );
        tableOracleResponse.getColumnModel().getSelectionModel().setSelectionMode( javax.swing.ListSelectionModel.SINGLE_INTERVAL_SELECTION );

        btnLoadFile.setText( "Load File" );
        btnLoadFile.addActionListener( new java.awt.event.ActionListener()
        {
            public void actionPerformed( java.awt.event.ActionEvent evt )
            {
                btnLoadFileActionPerformed( evt );
            }
        } );

        btnSaveFile.setText( "Save File" );
        btnSaveFile.addActionListener( new java.awt.event.ActionListener()
        {
            public void actionPerformed( java.awt.event.ActionEvent evt )
            {
                btnSaveFileActionPerformed( evt );
            }
        } );

        btnNewMsgs.setText( "New Messages" );
        btnNewMsgs.addActionListener( new java.awt.event.ActionListener()
        {
            public void actionPerformed( java.awt.event.ActionEvent evt )
            {
                btnNewMsgsActionPerformed( evt );
            }
        } );

        btnLoadServerKey.setText( "Load Cert" );
        btnLoadServerKey.setToolTipText( "DER-encoded-binary X.509 or Base-64-encoded X.509 (.CER)" );
        btnLoadServerKey.addActionListener( new java.awt.event.ActionListener()
        {
            public void actionPerformed( java.awt.event.ActionEvent evt )
            {
                btnLoadServerKeyActionPerformed( evt );
            }
        } );

        lbServerError.setFont( new java.awt.Font( "Tahoma", 1, 18 ) ); // NOI18N
        lbServerError.setText( "Server Error Table:" );

        lbTableRows.setText( "Table Rows:" );

        tbRowCount.setHorizontalAlignment( javax.swing.JTextField.CENTER );

        btnAbort.setText( "Abort" );
        btnAbort.setEnabled( false );
        btnAbort.addActionListener( new java.awt.event.ActionListener()
        {
            public void actionPerformed( java.awt.event.ActionEvent evt )
            {
                btnAbortActionPerformed( evt );
            }
        } );

        btnLoadPubKey.setText( "Load Key" );
        btnLoadPubKey.setEnabled( false );
        btnLoadPubKey.addActionListener( new java.awt.event.ActionListener()
        {
            public void actionPerformed( java.awt.event.ActionEvent evt )
            {
                btnLoadPubKeyActionPerformed( evt );
            }
        } );

        cbPKCS1GenWithEncData.setText( "<html><p>With EncryptedData</p></html>" );
        cbPKCS1GenWithEncData.setEnabled( false );
        cbPKCS1GenWithEncData.addActionListener( new java.awt.event.ActionListener()
        {
            public void actionPerformed( java.awt.event.ActionEvent evt )
            {
                cbPKCS1GenWithEncDataActionPerformed( evt );
            }
        } );

        cbIgnorePayResponse.setText( "<html><p>Ignore Payload <br>Response</p></html>" );
        cbIgnorePayResponse.addActionListener( new java.awt.event.ActionListener()
        {
            public void actionPerformed( java.awt.event.ActionEvent evt )
            {
                cbIgnorePayResponseActionPerformed( evt );
            }
        } );

        requestResponseSplitPane.setDividerLocation( 150 );
        requestResponseSplitPane.setOrientation( javax.swing.JSplitPane.VERTICAL_SPLIT );

        rTextScrollPane1.setHorizontalScrollBarPolicy( javax.swing.ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER );
        rTextScrollPane1.setFoldIndicatorEnabled( true );
        rTextScrollPane1.setHorizontalScrollBar( null );

        requestViewer.setEditable( false );
        requestViewer.setColumns( 20 );
        requestViewer.setRows( 5 );
        requestViewer.setSyntaxEditingStyle( "text/xml" );
        rTextScrollPane1.setViewportView( requestViewer );

        requestResponseSplitPane.setTopComponent( rTextScrollPane1 );

        rTextScrollPane2.setHorizontalScrollBarPolicy( javax.swing.ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER );
        rTextScrollPane2.setFoldIndicatorEnabled( true );
        rTextScrollPane2.setHorizontalScrollBar( null );

        responseViewer.setEditable( false );
        responseViewer.setColumns( 20 );
        responseViewer.setRows( 5 );
        responseViewer.setSyntaxEditingStyle( "text/xml" );
        rTextScrollPane2.setViewportView( responseViewer );

        requestResponseSplitPane.setRightComponent( rTextScrollPane2 );

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout( this );
        this.setLayout( layout );
        layout.setHorizontalGroup( layout.createParallelGroup( javax.swing.GroupLayout.Alignment.LEADING ).addGroup( layout.createSequentialGroup().addContainerGap().addGroup( layout.createParallelGroup( javax.swing.GroupLayout.Alignment.LEADING ).addComponent( lbServerError ).addGroup( layout.createSequentialGroup().addGroup( layout.createParallelGroup( javax.swing.GroupLayout.Alignment.LEADING ).addGroup( layout.createParallelGroup( javax.swing.GroupLayout.Alignment.LEADING,
                                                                                                                                                                                                                                                                                                                                                                                                                                                       false ).addComponent( btnNewMsgs,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                             javax.swing.GroupLayout.PREFERRED_SIZE,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                             0,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                             Short.MAX_VALUE ).addComponent( btnSaveFile,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                             javax.swing.GroupLayout.DEFAULT_SIZE,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                             javax.swing.GroupLayout.DEFAULT_SIZE,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                             Short.MAX_VALUE ).addComponent( btnLoadServerKey,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                             javax.swing.GroupLayout.DEFAULT_SIZE,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                             javax.swing.GroupLayout.DEFAULT_SIZE,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                             Short.MAX_VALUE ).addComponent( btnLoadPubKey,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                             javax.swing.GroupLayout.DEFAULT_SIZE,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                             javax.swing.GroupLayout.DEFAULT_SIZE,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                             Short.MAX_VALUE ).addComponent( cbPKCS1GenWithEncData,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                             javax.swing.GroupLayout.PREFERRED_SIZE,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                             0,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                             Short.MAX_VALUE ).addComponent( cbIgnorePayResponse ).addGroup( layout.createSequentialGroup().addComponent( lbTableRows ).addPreferredGap( javax.swing.LayoutStyle.ComponentPlacement.RELATED ).addComponent( tbRowCount ) ).addComponent( ServErrMsgProgressBar,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         javax.swing.GroupLayout.PREFERRED_SIZE,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         Short.MAX_VALUE ).addComponent( btnAbort,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         javax.swing.GroupLayout.DEFAULT_SIZE,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         javax.swing.GroupLayout.DEFAULT_SIZE,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         Short.MAX_VALUE ) ).addComponent( btnLoadFile,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                           javax.swing.GroupLayout.PREFERRED_SIZE,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                           145,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                           javax.swing.GroupLayout.PREFERRED_SIZE ) ).addPreferredGap( javax.swing.LayoutStyle.ComponentPlacement.RELATED ).addComponent( jScrollPane1,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          javax.swing.GroupLayout.PREFERRED_SIZE,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          145,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          javax.swing.GroupLayout.PREFERRED_SIZE ).addPreferredGap( javax.swing.LayoutStyle.ComponentPlacement.RELATED ).addComponent( requestResponseSplitPane,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       javax.swing.GroupLayout.DEFAULT_SIZE,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       146,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       Short.MAX_VALUE ) ) ).addContainerGap() ) );

        layout.linkSize( javax.swing.SwingConstants.HORIZONTAL, new java.awt.Component[] { btnLoadFile, btnSaveFile } );

        layout.setVerticalGroup( layout.createParallelGroup( javax.swing.GroupLayout.Alignment.LEADING ).addGroup( javax.swing.GroupLayout.Alignment.TRAILING,
                                                                                                                   layout.createSequentialGroup().addGap( 25,
                                                                                                                                                          25,
                                                                                                                                                          25 ).addComponent( lbServerError ).addGap( 15,
                                                                                                                                                                                                     15,
                                                                                                                                                                                                     15 ).addGroup( layout.createParallelGroup( javax.swing.GroupLayout.Alignment.LEADING ).addComponent( requestResponseSplitPane ).addGroup( layout.createSequentialGroup().addComponent( btnLoadFile ).addGap( 7,
                                                                                                                                                                                                                                                                                                                                                                                                                  7,
                                                                                                                                                                                                                                                                                                                                                                                                                  7 ).addComponent( btnSaveFile ).addGap( 7,
                                                                                                                                                                                                                                                                                                                                                                                                                                                          7,
                                                                                                                                                                                                                                                                                                                                                                                                                                                          7 ).addComponent( btnLoadServerKey ).addPreferredGap( javax.swing.LayoutStyle.ComponentPlacement.RELATED ).addComponent( btnLoadPubKey ).addGap( 3,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                           3,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                           3 ).addComponent( cbIgnorePayResponse,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                             javax.swing.GroupLayout.PREFERRED_SIZE,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                             javax.swing.GroupLayout.DEFAULT_SIZE,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                             javax.swing.GroupLayout.PREFERRED_SIZE ).addPreferredGap( javax.swing.LayoutStyle.ComponentPlacement.RELATED ).addComponent( cbPKCS1GenWithEncData,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          javax.swing.GroupLayout.PREFERRED_SIZE,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          33,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          javax.swing.GroupLayout.PREFERRED_SIZE ).addPreferredGap( javax.swing.LayoutStyle.ComponentPlacement.RELATED ).addComponent( btnNewMsgs ).addPreferredGap( javax.swing.LayoutStyle.ComponentPlacement.RELATED ).addComponent( ServErrMsgProgressBar,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        javax.swing.GroupLayout.PREFERRED_SIZE,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        6,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        javax.swing.GroupLayout.PREFERRED_SIZE ).addPreferredGap( javax.swing.LayoutStyle.ComponentPlacement.UNRELATED ).addGroup( layout.createParallelGroup( javax.swing.GroupLayout.Alignment.BASELINE ).addComponent( lbTableRows ).addComponent( tbRowCount,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      javax.swing.GroupLayout.PREFERRED_SIZE,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      javax.swing.GroupLayout.DEFAULT_SIZE,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      javax.swing.GroupLayout.PREFERRED_SIZE ) ).addPreferredGap( javax.swing.LayoutStyle.ComponentPlacement.UNRELATED ).addComponent( btnAbort ).addGap( 0,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          0,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          Short.MAX_VALUE ) ).addComponent( jScrollPane1,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                            javax.swing.GroupLayout.DEFAULT_SIZE,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                            512,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                            Short.MAX_VALUE ) ).addGap( 20,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        20,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        20 ) ) );

        cbIgnorePayResponse.getAccessibleContext().setAccessibleName( "<html><p>Ignore Payload Response</p></html>" );
    }// </editor-fold>//GEN-END:initComponents

    private void btnSaveFileActionPerformed( java.awt.event.ActionEvent evt )
    {// GEN-FIRST:event_SaveFileBtnActionPerformed

        chooseSaveFile.showSaveDialog( this );
    }// GEN-LAST:event_SaveFileBtnActionPerformed

    private void btnNewMsgsActionPerformed(java.awt.event.ActionEvent evt) {// GEN-FIRST:event_NewMsgsBtnActionPerformed
		if (btnNewMsgs.isEnabled()) {
			if (isErrorUserAttackConfiguration()) {
				return;
			}

			m_Option.deleteAllErrorTabDatas();
			try {
				m_Option.getServerBehaviour();
			} catch (IOException | CryptoAttackException | JAXBException | AbstractMethodError ex) {
				LOG.error(ex);
			}
		}
	}// GEN-LAST:event_NewMsgsBtnActionPerformed

    public boolean isErrorUserAttackConfiguration()
        throws HeadlessException
    {
        if ( null == m_Option.getAttackCfg().getChosenAttackPayload() )
        {
            JOptionPane.showMessageDialog( null, "Please choose an attack payload.", "No attack payload",
                                           JOptionPane.OK_OPTION );
            return true;
        }
        else if ( m_Option.getAttackCfg().getChosenAttackPayload() instanceof EncryptedKeyElement )
        {
            if ( XMLEncryptionAttackMode.PKCS1_ATTACK != m_Option.getAttackCfg().getXMLEncryptionAttack() )
            {
                JOptionPane.showMessageDialog( null, "EncryptedKey attack element but cbc attack mode",
                                               "Wrong attack mode", JOptionPane.OK_OPTION );
                return true;
            }
            else if ( null == m_Option.getAttackCfg().getPKCS1AttackCfg().getServerRSAPubKey() )
            {
                JOptionPane.showMessageDialog( null, "Please set the public key of server!", "Public Key not set",
                                               JOptionPane.OK_OPTION );
                return true;
            }
        }
        else if ( m_Option.getAttackCfg().getChosenAttackPayload() instanceof EncryptedDataElement )
        {
            if ( XMLEncryptionAttackMode.CBC_ATTACK != m_Option.getAttackCfg().getXMLEncryptionAttack() )
            {
                JOptionPane.showMessageDialog( null, "EncryptedData attack element but pkcs1 attack mode",
                                               "Wrong attack mode", JOptionPane.OK_OPTION );
                return true;
            }
        }
        return false;
    }

    private void btnLoadFileActionPerformed( java.awt.event.ActionEvent evt )
    {// GEN-FIRST:event_LoadFileBtnActionPerformed
        chooseLoad.showOpenDialog( this );
    }// GEN-LAST:event_LoadFileBtnActionPerformed

    private void btnLoadServerKeyActionPerformed( java.awt.event.ActionEvent evt )
    {// GEN-FIRST:event_LoadServerKeyBtnActionPerformed
        chooseLoadCert.showOpenDialog( this );
    }// GEN-LAST:event_LoadServerKeyBtnActionPerformed

    private void chooseLoadActionPerformed( java.awt.event.ActionEvent evt )
    {// GEN-FIRST:event_LoadChooserActionPerformed
        int returnVal = chooseLoad.getApproveButtonMnemonic();
        File file = chooseLoad.getSelectedFile();
        if ( returnVal == JFileChooser.APPROVE_OPTION && null != file )
        {
            m_Option.LoadDataFromFile( new File( file.getAbsolutePath() ) );
            m_EncryptionPlugin.checkState();
        }
    }// GEN-LAST:event_LoadChooserActionPerformed

    private void chooseSaveFileActionPerformed(java.awt.event.ActionEvent evt) {// GEN-FIRST:event_SaveFileChooserActionPerformed
		int returnVal = chooseSaveFile.getApproveButtonMnemonic();
		File file = chooseSaveFile.getSelectedFile();
		if (returnVal == JFileChooser.APPROVE_OPTION && null != file) {
			try {
				m_Option.SaveDataToFile(file);
			} catch (IOException | JAXBException ex) {
				LOG.error(ex);
			}
			initTable();
		}
	}// GEN-LAST:event_SaveFileChooserActionPerformed

    private void chooseLoadCertActionPerformed(java.awt.event.ActionEvent evt) {// GEN-FIRST:event_LoadCertChooserActionPerformed
		int returnVal = chooseLoadCert.getApproveButtonMnemonic();
		File file = chooseLoadCert.getSelectedFile();
		if (returnVal == JFileChooser.APPROVE_OPTION && null != file) {
			try {
				m_Option.setCertServer(file);
				cbPKCS1GenWithEncData.setEnabled(true);
				// m_EncryptionPlugin.checkState();
			} catch (IOException | CertificateException | InvalidKeySpecException | NoSuchAlgorithmException | NoSuchProviderException ex) {
				LOG.error(ex);
			}
		}
	}// GEN-LAST:event_LoadCertChooserActionPerformed

    private void btnAbortActionPerformed( java.awt.event.ActionEvent evt )
    {// GEN-FIRST:event_AbortButnActionPerformed
        m_Option.stopSendingMessages();
    }// GEN-LAST:event_AbortButnActionPerformed

    private void btnLoadPubKeyActionPerformed( java.awt.event.ActionEvent evt )
    {// GEN-FIRST:event_LoadPubKeyBtnActionPerformed
        chooseLoadPubKey.showOpenDialog( this );
    }// GEN-LAST:event_LoadPubKeyBtnActionPerformed

    private void chooseLoadPubKeyActionPerformed(java.awt.event.ActionEvent evt) {// GEN-FIRST:event_LoadPubKeyChooserActionPerformed
		int returnVal = chooseLoadPubKey.getApproveButtonMnemonic();
		File file = chooseLoadPubKey.getSelectedFile();
		if (returnVal == JFileChooser.APPROVE_OPTION && null != file) {
			try {
				m_Option.setPubKeyServer(file);
				cbPKCS1GenWithEncData.setEnabled(true);
				// m_EncryptionPlugin.checkState();
			} catch (IOException | CertificateException | InvalidKeySpecException | NoSuchAlgorithmException | NoSuchProviderException ex) {
				LOG.error(ex);
			}
		}
	}// GEN-LAST:event_LoadPubKeyChooserActionPerformed

    private void cbPKCS1GenWithEncDataActionPerformed( java.awt.event.ActionEvent evt )
    {// GEN-FIRST:event_PKCS1GenWithEncDataActionPerformed
        if ( null != m_Option && null != m_EncryptionPlugin )
        {
            m_Option.setIsPKCS1WithEncData( cbPKCS1GenWithEncData.isSelected() );
        }

    }// GEN-LAST:event_PKCS1GenWithEncDataActionPerformed

    private void cbIgnorePayResponseActionPerformed( java.awt.event.ActionEvent evt )
    {// GEN-FIRST:event_IgnorePayResponseActionPerformed
        if ( null != m_Option && null != m_EncryptionPlugin )
        {
            m_Option.setIsIgnoreRequestResponse( cbIgnorePayResponse.isSelected() );
        }
    }// GEN-LAST:event_IgnorePayResponseActionPerformed

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JProgressBar ServErrMsgProgressBar;

    private javax.swing.JButton btnAbort;

    private javax.swing.JButton btnLoadFile;

    private javax.swing.JButton btnLoadPubKey;

    private javax.swing.JButton btnLoadServerKey;

    private javax.swing.JButton btnNewMsgs;

    private javax.swing.JButton btnSaveFile;

    private javax.swing.JCheckBox cbIgnorePayResponse;

    private javax.swing.JCheckBox cbPKCS1GenWithEncData;

    private javax.swing.JFileChooser chooseLoad;

    private javax.swing.JFileChooser chooseLoadCert;

    private javax.swing.JFileChooser chooseLoadPubKey;

    private javax.swing.JFileChooser chooseSaveFile;

    private javax.swing.JScrollPane jScrollPane1;

    private javax.swing.JLabel lbServerError;

    private javax.swing.JLabel lbTableRows;

    private org.fife.ui.rtextarea.RTextScrollPane rTextScrollPane1;

    private org.fife.ui.rtextarea.RTextScrollPane rTextScrollPane2;

    private javax.swing.JSplitPane requestResponseSplitPane;

    private org.fife.ui.rsyntaxtextarea.RSyntaxTextArea requestViewer;

    private org.fife.ui.rsyntaxtextarea.RSyntaxTextArea responseViewer;

    private javax.swing.JTable tableOracleResponse;

    private javax.swing.JTextField tbRowCount;

    // End of variables declaration//GEN-END:variables

    @Override
    public void bindingDoUnbind()
    {
        // throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose
        // Tools | Templates.
    }

    @Override
    public AbstractOption getUsedOption()
    {
        return m_Option;
    }

    public void updateProgressBar( int progressStep )
    {
        ServErrMsgProgressBar.setValue( progressStep );
        ServErrMsgProgressBar.repaint();
    }

    private void updateRowHeights()
    {
        final int rowCount = tableOracleResponse.getRowCount();
        int rowHeight = tableOracleResponse.getRowHeight();
        final int columnCount = tableOracleResponse.getColumnCount();
        for ( int row = 0; row < rowCount; row++ )
        {
            for ( int column = 0; column < columnCount; column++ )
            {
                try
                {
                    Component comp =
                        tableOracleResponse.prepareRenderer( tableOracleResponse.getCellRenderer( row, column ), row,
                                                             column );
                    rowHeight = Math.max( rowHeight, comp.getPreferredSize().height );
                }
                catch ( Exception e )
                {
                    LOG.error( "### Error: This should not happen, catched " + e.getMessage() );
                }

                tableOracleResponse.setRowHeight( row, rowHeight );
            }
        }
    }

    private void fixWidth( final int columnIndex, final int width )
    {
        TableColumn column = tableOracleResponse.getColumnModel().getColumn( columnIndex );
        column.setMinWidth( width );
        column.setMaxWidth( width );
        column.setPreferredWidth( width );
    }

    public void initTable()
    {
        OracleResponseTableModel oracleResponseTableModel = new OracleResponseTableModel( m_Option.getResponseData() );
        tableOracleResponse.setModel( oracleResponseTableModel );
        tableOracleResponse.setDefaultRenderer( String.class, new MultiLinesCellRenderer() );
        JComboBox resultCombo = new JComboBox( Result.values() );
        resultCombo.setRenderer( new ResultTableCellRenderer() );
        tableOracleResponse.getColumnModel().getColumn( COLUMN_IDX_ORACLERESULT ).setCellEditor( new DefaultCellEditor(
                                                                                                                        resultCombo ) );
        DefaultTableCellRenderer rend = new DefaultTableCellRenderer();
        tableOracleResponse.getColumnModel().getColumn( COLUMN_IDX_ORACLERESULT ).setCellRenderer( rend );
        tableOracleResponse.getTableHeader().setReorderingAllowed( false );
        // updateRowHeights();
        fixWidth( COLUMN_IDX_ROW, 40 );
        fixWidth( COLUMN_IDX_ORACLERESULT, 100 );
        tbRowCount.setText( String.valueOf( tableOracleResponse.getRowCount() ) );
        requestViewer.setText( "" );
        responseViewer.setText( ( "" ) );
    }

    public void updateTable( OracleResponse oracResp )
    {
        OracleResponseTableModel tabModel = (OracleResponseTableModel) tableOracleResponse.getModel();
        tabModel.update();
        // updateRowHeights();
        tbRowCount.setText( String.valueOf( tableOracleResponse.getRowCount() ) );
        fixWidth( COLUMN_IDX_ROW, 40 );
        fixWidth( COLUMN_IDX_ORACLERESULT, 100 );
    }

    public void setBtnsState( boolean state )
    {
        btnNewMsgs.setEnabled( state );
        btnLoadFile.setEnabled( state );
        btnSaveFile.setEnabled( state );
        btnLoadServerKey.setEnabled( state );
        btnAbort.setEnabled( !state );
        // LoadPubKeyBtn.setEnabled(state);

        if ( state )
        {
            m_EncryptionPlugin.checkState();
            ServErrMsgProgressBar.setValue( 0 );
        }
    }
}
